import torch
import torch.nn as nn

class BRNN(nn.Module):
    def __init__(self, num_classes, input_size, hidden_size, num_layers=1):
        super().__init__()
        self.rnn = nn.RNN(
            input_size=input_size,
            hidden_size=hidden_size,
            num_layers=num_layers,
        )
        self.fc = nn.Linear(hidden_size, num_classes)

    def forward(self, x):
        # x [batch_size, input_size, seq_len]
        x = x.permute(2, 0, 1)
        # x [seq_len, batch_size, input_size]
        out, h = self.rnn(x) # out: [seq_len, batch_size, hidden_size]
        # 取最后一个时间步的输出作为整个序列的表示
        out = out[-1]
        out = self.fc(out)
        return out

if __name__ == '__main__':
    model = BRNN(num_classes=11, input_size=2, hidden_size=512, num_layers=4)
    x = torch.randn(64, 2, 128) # 输入形状 [batch_size, input_size, seq_len]
    y = model(x)
    print(y.shape)


class BStopByLoss:
    def __init__(self, rounds, delta=0.01):
        '''
        连续rounds次, train_loss - min_train_loss > delta, 则停止训练
        '''
        self.rounds = rounds
        self.delta = delta
        self.min_trainLoss = float('inf')
        self.cnt = 0
    def __call__(self, train_loss):
        if train_loss >= self.min_trainLoss + self.delta:
            self.cnt += 1
        if train_loss < self.min_trainLoss:
            self.min_trainLoss = train_loss
            self.cnt = 0
        if self.cnt > self.rounds:
            return True
        return False


class BStopByAccDelta:
    def __init__(self, rounds, delta=0.003):
        '''
        连续rounds次, |before_acc - val_acc| <= delta, 则停止训练
        '''
        self.rounds = rounds
        self.delta = delta
        self.before_acc = 0
        self.cnt = 0
    def __call__(self, val_acc):
        if -self.delta <= (val_acc - self.before_acc) <= self.delta:
            self.cnt += 1
            self.before_acc = val_acc
        elif self.cnt > 0:
            self.cnt -= 1
            self.before_acc = val_acc
        else:
            self.before_acc = val_acc

        if self.cnt > self.rounds:
            return True
        return False


import subprocess
def BRunCmd(*args, show=True):
    '''
    可传入多个字符串, 在cmd中运行
    :param args:
    :param show: 若show=True, 则会单开一个cmd, 在cmd中运行
    :return:
    '''
    command = ''
    for i in range(len(args)):
        if i == len(args) - 1:
            command += str(args[i])
            break
        command += str(args[i]) + ' && '
    if show:
        command = f'start cmd /K "{command}"'
    # print(command)
    subprocess.run(command, shell=True)

def BRunPython(*args):
    '''
    可传入多个字符串, 在当前python环境下运行
    :param args: 以python开头, 用于运行.py文件
    :param show:
    :return:
    '''
    for string in args:
        command_lst = string.split(' ')
        subprocess.run(command_lst)

    print("BRunPython结束:")
    for string in args:
        print("\t"+string)

if __name__ == '__main__':
    BRunCmd("echo hello","echo world","echo awa", show=True)
    # BRunPython("python E:\\byzh_workingplace\\test1.py")